import sys, os, traceback
import plesk_config, osutil, subproc, agents_pubkey, log, dirutil
import GPG


__TEMP_GPGDIR = "/tmp/gpgdir"


class SystemError(Exception):
    def __init__(self, desc):
        self.desc = desc
    def __str__(self):
        return self.desc


class WrongSignature(Exception):
    pass


def usage():
    print "Usage: agent-extract <signed-archive-name> <destination-directory>"


def main():
    if len(sys.argv) != 3:
        usage()
        sys.exit(1)

    archiveFile = unicode(sys.argv[1], "utf8")
    sessionDir = unicode(sys.argv[2], "utf8")

    try:
        dirutil.createNewDirByAbsoluteAsciiPath(sessionDir)
    except OSError, e:
        sys.stdout.write(unicode(e))
        sys.exit(1)

    log.initLog("agent-extract", sessionDir)

    gpg_home = plesk_config.get("PRODUCT_ROOT_D") + __TEMP_GPGDIR + unicode(os.getpid())
    gpg_filename = plesk_config.get("GPG_BIN")

    try:
        os.makedirs(gpg_home)
        try:
            gpg = GPG.GPG(gnupghome = gpg_home, gpgbinary = gpg_filename)

            res = gpg.import_key(agents_pubkey.agents_pubkey)
            if not res:
                raise SystemError("Unable to import signing key: " + res.summary())

            signed_archive = file(archiveFile).read()
            res = gpg.verify(signed_archive)
            if not res:
                raise WrongSignature

            archive = gpg.decrypt(signed_archive)
            if not archive:
                raise SystemError("Unable to extract archive from signed bundle")

            tar = plesk_config.get("TAR_BIN")
            cmd = subproc.CmdLine(tar, args = ["-x", "--no-same-owner", "-C", sessionDir, "-f", "-"])
            cmd.addstdin(str(archive))
            cmd.spawn()
        finally:
            osutil.unlink_recursively(gpg_home)
    except WrongSignature:
        sys.stdout.write("File is not properly signed.\n")
        sys.exit(2)
    except SystemError, e:
        sys.stdout.write(unicode(e) + "\n")
        sys.exit(1)
    except Exception, e:
        traceback.print_exc(None, sys.stdout)
        sys.stdout.write("\n")
        sys.exit(1)


if __name__ == '__main__':
    main()
